/*
    Ring (circular) buffer implementation.
    Author: Kalim Moghul (kfm9 at cornell dot edu)
    Invariants:
        head==tail iff buffer is empty.
        If buffer is not empty, head points at next valid element to be consumed.
        tail always points at the next empty element.
            => There is always one unused element in a full buffer.
            => length must be greater than 1
    */

typedef unsigned int rbdatatype;       //define storage type of ringbuffer

typedef struct ringbuf
{
    rbdatatype *buf;              //points to data array
    unsigned char length;            //length of data array
    unsigned char head, tail;        //producer and consumer indices
} ringbuffer;

//initializes the given ringbuffer with the supplied array and its length
inline void rbinit(ringbuffer *rb, rbdatatype *array, unsigned char length);

//returns boolean true if the ringbuffer is empty, false otherwise
inline unsigned char rbisempty(ringbuffer *rb);

//returns boolean true if the ringbuffer is full, false otherwise
inline unsigned char rbisfull(ringbuffer *rb);

//consumes an element from the buffer
//returns NULL if buffer is empty or a pointer to the array element otherwise
inline rbdatatype* rbget(ringbuffer *rb);

//puts an element into the buffer
//returns 0 if buffer is full, otherwise returns 1
inline unsigned char rbput(ringbuffer *rb, rbdatatype c);

#include "ringbuffer.c"